import { Entity, Column, JoinColumn, PrimaryGeneratedColumn, OneToMany, ManyToOne, OneToOne, ManyToMany } from 'typeorm';
import { BasicREntity } from '../basicR.type';
import { IsNumber, IsNotEmpty, IsOptional, IsArray } from 'class-validator';
import { User } from '../app/user.type';
import { Farm } from './farm.type';
import { Visit } from './visit.type';

export enum FarmRole {
    OWNER = 1,
    CONSULTANT = 2,
    CONSULTANT_AUX = 3,
    WORKER = 4
}

@Entity()
export class UserFarmRole extends BasicREntity {

    @IsNotEmpty()
    @ManyToOne(type => User, user => user.userFarmRoles, { nullable: false })
    user: User;

    @IsNotEmpty()
    @ManyToOne(type => Farm, farm => farm.userFarmRoles, { nullable: false })
    farm: Farm;

    @IsNumber()
    @Column({ nullable: false, type: 'int' })
    role: FarmRole;

    // Consultants

    @IsOptional()
    @ManyToOne(type => UserFarmRole, userFarmRole => userFarmRole.supervisor, { nullable: true })
    supervisor?: UserFarmRole;
    
    // TODO: Add school

    @IsOptional()
    @IsArray()
    @OneToMany(type => Visit, visit => visit.consultant, { nullable: true })
    visits?: Visit[];
}